
use strict;

# P(C|D) = P(C|W1)P(C|...)P(C|Wn) = P(W...|C)P(C) / P(W...)

# trainig calculates P(C), where C is spam/ham, e.g.: P(spam) vs P(ham) 
# and P(W|C) for each Word; e.g. P(crypto|spam), P(crypto|ham) ... etc.

# Bayes: P(C|w1,w2,w3, wn) = P(w1,w2,..,wn|C)P(C) / P(w1,w2,..,wn)
# naive bayes:  P(C|w1,w2,w3, wn) =  P(C|w1) P(C|w2) P(C|w3) P(C|wn)

my $catcnttot;
my %catcnt;
my %model;
my %modeltot;

sub train {
  my ($cat) = @_;
  for my $f (glob "$cat/*.txt"){
    $catcnt{$cat}++;   # count categories. e.g. P(C)
    $catcnttot++;
    open my $in, $f or die $!;
    local $/=undef;
    $_ = <$in>;
    my @words = map { lc } split/\s+/;
    for my $w (@words){
      $model{$cat}{$w}++;     # P(w|cat) = $model{$cat}{$w}/$modeltot{$cat}
      $modeltot{$cat}++;
    }
    close $in;
  }
}


sub classify {
  for my $f (glob "test/*.txt"){
    my %r = ();
    open my $in, $f or die $!;
    local $/=undef;
    $_ = <$in>;
    my @words = map { lc } split/\s+/;
    close $in;
    for my $cat (qw(spam ham)) {
       # sums of logs (instead of product of probabilities)
       $r{$cat} = log( $catcnt{$cat}/$catcnttot );  # P(C)
       for my $w (@words){
         # if word is not found, then add a very very low probability.
         if(defined $model{$cat}{$w}){ 
           $r{$cat} += log( $model{$cat}{$w}/$modeltot{$cat}  );
         }else{
           $r{$cat} += log(0.001);
         }
       }
    }
    for(qw(spam ham)){
      $r{$_} = exp($r{$_});   # convert back to probabilities.
    }
    my $tot = $r{spam} + $r{ham};
    $r{spam} /= $tot; $r{ham} /= $tot;
    print "$f: spam=$r{spam}; ham=$r{ham}\n";
  }
}

train("spam");
train("ham");
classify();


